import { License } from '../integration/index.js';
import { Request, UrlTools } from '../util/index.js';
import { AssistantInstaller, DockerAssistant } from './index.js';
class Login {
    constructor() {
        this.defaultProcotol = 'http';
    }
    isValidUser(user) {
        if (!user || !user.ID)
            throw new Error('INVALID_USER');
    }
    async getToken(serverHost, { CDOPERADOR, NMOPERADOR, NRORG }) {
        const dns = UrlTools.getUrlWithProtocol(serverHost);
        const route = `${dns}:${NRORG}/API/generateToken`;
        const response = await Request.retryCall(route, {
            method: 'POST',
            data: { CDOPERADOR, NMOPERADOR },
        });
        return response.token;
    }
    async testDbConnection(serverHost, orgId) {
        const dns = UrlTools.getUrlWithProtocol(serverHost);
        const route = `${dns}:${orgId}/API/dbConnection`;
        const response = await Request.retryCall(route, { method: 'GET' });
        return response;
    }
    async getAccessProfile(serverHost, { CDOPERADOR, NRORG }) {
        const dns = UrlTools.getUrlWithProtocol(serverHost);
        const route = `${dns}:${NRORG}/API/accessProfile`;
        const accessProfile = await Request.post(route, { CDOPERADOR });
        return accessProfile;
    }
    async setServerForOrganization(params) {
        return License.setServerForOrganization(params);
    }
    async login({ email, password, agent, agentVersion }) {
        let errorInfo = {};
        const defaultErrorHandler = (e, errorCode) => {
            errorInfo = {
                message: e.message,
                stack: e.stack,
            };
            throw new Error(errorCode);
        };
        try {
            const response = await License.auth({ email, password }).catch((e) => {
                defaultErrorHandler(e, 'ERR_CALL_LICENSE');
            });
            if (!response.dataset) {
                errorInfo = {
                    message: response.error || response.message || 'Unexpected Error',
                    stack: 'call license',
                    data: response,
                };
                throw new Error('ERR_CALL_LICENSE');
            }
            const { dataset: { operador } } = response;
            const { NRORG, ID: CDOPERADOR, NAME: NMOPERADOR } = operador;
            if (NRORG && NRORG.toString() === '20') {
                throw new Error('ERR_UNAVAILABLE_ORG_20');
            }
            this.isValidUser(operador);
            const { dataset: { serverUrl: server } } = await License.getServerByOrganization({ NRORG });
            const { SERVER_DNS = '', NMORGANIZACAO = '' } = server;
            let token = '';
            const ACCESS_PROFILE = { PERMISSIONS: '', error: false, errorMessage: '' };
            const serverInfo = await DockerAssistant.handleClientServer(NRORG, SERVER_DNS).catch((e) => {
                defaultErrorHandler(e, 'ERR_CREATE_CONTAINER_ORG');
            });
            if (!SERVER_DNS && serverInfo.serverName) {
                await this.setServerForOrganization({
                    NRORG,
                    HOST: serverInfo.host,
                });
            }
            if (agent === 'assistant-installer') {
                AssistantInstaller.validateAgentVersion(agentVersion);
            }
            else {
                await this.testDbConnection(SERVER_DNS || serverInfo.host, NRORG).catch((e) => {
                    defaultErrorHandler(e, 'ERR_CONNECTION_DB');
                });
                token = await this.getToken(SERVER_DNS || serverInfo.host, { CDOPERADOR, NMOPERADOR, NRORG }).catch((e) => {
                    defaultErrorHandler(e, 'ERR_GENERATE_TOKEN');
                });
                await this.getAccessProfile(SERVER_DNS || '', { CDOPERADOR }).then((accessProfile) => {
                    Object.assign(ACCESS_PROFILE, accessProfile);
                }).catch((e) => {
                    ACCESS_PROFILE.error = true;
                    ACCESS_PROFILE.errorMessage = e.message;
                });
            }
            return {
                STATUS: true,
                STATUS_MSG: 'SUCCESS',
                STATUS_DATA: {},
                NODE: SERVER_DNS || serverInfo.host,
                NRORG,
                NMORGANIZACAO,
                TOKEN: token,
                CDOPERADOR,
                NMOPERADOR,
                ACCESS_PROFILE,
            };
        }
        catch (error) {
            return {
                STATUS: false,
                STATUS_MSG: error.message,
                STATUS_DATA: errorInfo,
            };
        }
    }
}
const instance = new Login();
export { instance as Login };
export default instance;
