import path from 'path';
import { PrismaClient } from '../db/index.js';
import { File, Dependency } from '../util/index.js';
class Product {
    async create(params) {
        const { devopsId, productName, requiredProducts, description = '', productImage = '', requirements = '', guideUrl = '', } = params;
        const data = {
            devopsId,
            productName,
            description,
            productImage,
            requirements,
            guideUrl,
            requiredProducts: {
                connect: requiredProducts?.map((productId) => ({ id: productId })),
            },
        };
        return PrismaClient.product.create({ data, include: { requiredProducts: true } });
    }
    async list() {
        return PrismaClient.product.findMany({
            include: { requiredProducts: true },
        });
    }
    async listByDevopsId(devopsIdCollection) {
        if (!devopsIdCollection)
            throw new Error('Invalid devopsIdCollection');
        return PrismaClient.product.findMany({
            where: { devopsId: { in: devopsIdCollection } },
            include: { requiredProducts: true },
        });
    }
    async removeProductImage(fileName) {
        if (!process.env.IMAGES_PATH) {
            throw new Error('Invalid IMAGES_PATH');
        }
        const filePath = path.join(process.env.IMAGES_PATH, fileName);
        await new File(fileName, filePath).remove();
    }
    async remove({ productsToRemove }) {
        const products = Array.isArray(productsToRemove) ? productsToRemove : [];
        const where = { devopsId: { in: products.map((product) => product.devopsId) } };
        const result = await PrismaClient.product.findMany({
            where: {
                requiredProducts: {
                    some: {
                        id: {
                            in: productsToRemove?.map((product) => product.id),
                        },
                    },
                },
            },
        });
        if (result && result.length) {
            return {
                error: 'PRODUCT_HAS_DEPENDENTS',
                data: result,
            };
        }
        const response = PrismaClient.product.deleteMany({ where });
        const promiseRemoveImages = [];
        productsToRemove?.forEach((product) => {
            if (product.productImage) {
                promiseRemoveImages.push(this.removeProductImage(product.productImage));
            }
        });
        await Promise.allSettled(promiseRemoveImages);
        return response;
    }
    async hasCircularDependency(updatedProduct) {
        const { devopsId, requiredProducts } = updatedProduct;
        if (!Array.isArray(requiredProducts) || !requiredProducts.length) {
            return { result: false, circularDependency: [] };
        }
        const allProducts = await this.list();
        const dependencyList = allProducts.reduce((dependency, product) => {
            if (product.devopsId.toString() === devopsId.toString()) {
                dependency[product.id] = requiredProducts;
            }
            else {
                dependency[product.id] = product.requiredProducts.map((requiredProduct) => requiredProduct.id);
            }
            return dependency;
        }, {});
        const circularDependency = Dependency.findCircularDependencies(dependencyList);
        if (circularDependency && circularDependency.length) {
            const [firstError] = circularDependency;
            return {
                result: true,
                circularDependency: allProducts.filter((product) => firstError.includes(product.id)),
            };
        }
        return {
            result: false,
            circularDependency: [],
        };
    }
    async modify(params) {
        const { devopsId, requiredProducts: productsId, ...otherParams } = params;
        let requiredProducts;
        if (Array.isArray(productsId)) {
            const onlyIds = productsId.every((item) => typeof item === 'string');
            requiredProducts = {
                set: onlyIds ? productsId.map((id) => ({ id })) : productsId.map((product) => ({ id: product.id })),
            };
        }
        const productDependencyResult = await this.hasCircularDependency({
            devopsId,
            requiredProducts: productsId,
        });
        if (productDependencyResult.result) {
            const { circularDependency } = productDependencyResult;
            return { circularDependency };
        }
        await PrismaClient.product.update({
            where: { devopsId: Number(devopsId) },
            data: { ...otherParams, requiredProducts },
        });
        const savedRow = await PrismaClient.product.findUnique({
            where: { devopsId },
            include: { requiredProducts: true },
        });
        return savedRow;
    }
}
const instance = new Product();
export { instance as Product };
export default instance;
