import { PrismaClient } from '../db/index.js';
import { Gzip } from '../util/index.js';
class ProductSetting {
    encodeConfig(config = {}) {
        return Gzip.encode(JSON.stringify(config, null, 2));
    }
    async create(params) {
        const { devopsId, config } = params;
        const { INTERNAL_PROPS = {} } = config;
        delete config.INTERNAL_PROPS;
        const data = {
            devopsId: Number(devopsId),
            config: this.encodeConfig(config) || '',
            detail: {
                create: {
                    devopsId: Number(devopsId),
                    config: this.encodeConfig(INTERNAL_PROPS) || '',
                },
            },
        };
        const createdResponse = await PrismaClient.productSetting.create({ data, include: { detail: true } });
        const productResponse = await PrismaClient.product.findFirst({
            where: { devopsId: Number(devopsId) },
        });
        createdResponse.config = JSON.parse(Gzip.decode(createdResponse.config));
        return {
            productName: productResponse?.productName,
            ...createdResponse,
        };
    }
    async list(options) {
        const { decodeGZip = true, } = options || {};
        const data = await PrismaClient.productSetting.findMany({
            include: { product: true },
        });
        return data.map(({ config, product }) => {
            const configValue = decodeGZip ? JSON.parse(Gzip.decode(config)) : config;
            return ({
                config: configValue,
                devopsId: product.devopsId,
                productName: product.productName,
            });
        });
    }
    async find(devopsId, options) {
        const { decodeGZip = true, } = options || {};
        const data = await PrismaClient.productSetting.findUnique({
            where: { devopsId: Number(devopsId) },
            include: {
                detail: true,
            },
        });
        if (decodeGZip && data && data.config) {
            data.config = JSON.parse(Gzip.decode(data.config));
            if (data.detail && data.detail.config) {
                data.detail = JSON.parse(Gzip.decode(data.detail.config));
            }
        }
        return data;
    }
    async remove({ devopsId, productsToRemove }) {
        const products = Array.isArray(productsToRemove) ? productsToRemove : [];
        const where = { devopsId: { in: products.map(Number) } };
        if (devopsId) {
            const response = await PrismaClient.productSetting.deleteMany({
                where: { devopsId: Number(devopsId) },
            });
            return response;
        }
        if (Array.isArray(productsToRemove)) {
            const response = PrismaClient.productSetting.deleteMany({ where });
            return response;
        }
    }
    async modify(params) {
        const { devopsId, config } = params;
        const { INTERNAL_PROPS = {} } = config;
        delete config.INTERNAL_PROPS;
        const [response] = await PrismaClient.$transaction([
            PrismaClient.productSetting.update({
                where: { devopsId: Number(devopsId) },
                data: { config: this.encodeConfig(config) || '' },
            }),
            PrismaClient.productSettingDetail.update({
                where: { devopsId: Number(devopsId) },
                data: { config: this.encodeConfig(INTERNAL_PROPS) || '' },
            }),
        ]);
        const productResponse = await PrismaClient.product.findFirst({
            where: { devopsId: Number(devopsId) },
        });
        response.config = JSON.parse(Gzip.decode(response.config));
        return {
            productName: productResponse?.productName,
            ...response,
        };
    }
    async getSettingForApi() {
        const productSetting = await this.list({ decodeGZip: false });
        return productSetting;
    }
}
const instance = new ProductSetting();
export { instance as ProductSetting };
export default instance;
